shRNA_filter <- function(...)		 
{
	args <- list(...)
	select <- NULL
	##### below needed for wrapping in GP #####
	for(i in 1:length(args)) 
		{
		flag <- substring(args[[i]], 0, 2)
		value <- substring(args[[i]], 3, nchar(args[[i]]))
		if(flag=='-g')
			{
			gct.file <- value;
			}else if(flag=='-c') {
			chip.file<- value
			}else if(flag=='-h') {
			hplist<- value
			}else if(flag=='-r') {
			rmna<- value
			}else if(flag=='-e') {
			extension<- value
			}else if(flag=='-l') {
			libdir <- value
			}else {
			stop("ERROR: input option not recognized!")
			}
		}
	#############################################

### Debugging flags:
debug.skip.regression <- FALSE
debug.verbose.output <- TRUE
if (debug.verbose.output) { paste("Hello world: ", date()) }
###----------------------------------------------------------------------------
### function dprint()
###----------------------------------------------------------------------------
### Outputs its arguments in an informative way, but only if the global flag
### `debug.verbose.output` is set.
dprint <- function(...) {
  if (debug.verbose.output) {
    print(paste(collapse="",
                  c("DEBUG: ", format(Sys.time(), "%H:%M:%S: "), ...)))
  }
}


### load relevant libraries or install if needed
	dprint("Loading libraries.")
	source(paste(libdir, "common2.R", sep=''))

	setLibPath(libdir)

### Read chip file.
	dprint("Reading chip file.")
	readChipFile <- function() {
		df <- read.delim(chip.file, stringsAsFactors=FALSE)
		#hairpins <- gsub("\\w_\\w*", "", df[,1])
		hairpins <- df[,1]
		geneIds <- df[,3]
		type = colnames(df[1])
		retval <- data.frame(stringsAsFactors = FALSE,
				geneIds = df[,3],
				#hairpins = gsub("\\w_\\w*", "", df[,1]),
				hairpins = df[,1],
				geneSymbols = df[,2] ,
				type = colnames(df[1]))		
}
chipFrame <- readChipFile()

dprint("Reading gct file.")
### Read gct file 
	dataset <- MSIG.Gct2Frame(filename = gct.file)
###m2 has the actual scores from the gct file
	m2 <- data.matrix(dataset$ds)
	hairpins <- dataset$row.names


### Read in non-overlapping hairpins, check if by TRCN id or 21mer sequence
	dprint("Reading non-overlapping shRNA list.")
	nonoverlap <- read.delim(hplist, header=F, as.is=T)
	if (chipFrame$type[1] == "Target.Sequence"  )  {
	nonoverlap <- nonoverlap$V4
	hairpins.suffixFree<-hairpins
	} else {	
	hairpins.suffixFree = gsub("\\w_\\w*", "", hairpins) 
	nonoverlap <- nonoverlap$V2 }	

### Subset for the list of shRNAs that don't overlap in sequence
	dprint("Start excluding overlapping shRNAs.")
	flist <- match(nonoverlap,hairpins.suffixFree)
	#flist <- sort(match(nonoverlap,hairpins.suffixFree))
	m3<-dataset$ds[flist,]
	hairpins.suffixFree<-hairpins.suffixFree[flist]
	dprint("Done excluding overlapping shRNAs.")

### Add data from .chip annotation file
	x1<-merge(chipFrame, m3, by.x="hairpins", by.y="row.names", all.y=T)
	x1<-x1[,-c(2,4)]
	x<-{}
	gct<-{}
    x$row.descriptions<-x1[,2]
	x$data<-x1[,3:dim(x1)[2]]	
	x$rows<-x1[,1]
	##remove NA rows, if any  ### make this optional #really just want to remove NA genes, not NA data
	#if (rmna == "yes")  {
	#tlist <- which(!is.na(x$data[,1]))
	tlist <- which(!is.na(x$row.descriptions))
	#}  else  {
	#tlist <- 1:length(x$data[,1]) }
	gct$data<-x$data[tlist,]
	gct$row.descriptions<-x$row.descriptions[tlist]
	gct$rows<-x$rows[tlist]
		
### Make rownames unique
	gct$rows<- paste(gct$rows, gct$row.descriptions, sep='_')

### Remove any remaining duplicates (gene_seq pairs that are duplicated in Chip file with different GeneIds)
	xrows<-match(unique(gct$rows),gct$rows)
	gct$data<-gct$data[xrows,]
	gct$row.descriptions<-gct$row.descriptions[xrows]
	gct$rows<-gct$rows[xrows]
	
### Write out .gct file	
	filename<-paste(extension, "mapped_FC.rnai.gct", sep='_')
	rows <- dim(gct$data)[1]
	columns <- dim(gct$data)[2]
	m <- cbind(gct$rows, gct$row.descriptions, gct$data)
	f <- file(filename, "w")
	on.exit(close(f))
	cat("#1.2", "\n", file=f, append=TRUE, sep="")
	cat(rows, "\t", columns, "\n", file=f, append=TRUE, sep="")
	cat("Name", "\t", file=f, append=TRUE, sep="")
	cat("Description", file=f, append=TRUE, sep="")
	names <- colnames(gct$data)
	for(j in 1:length(names)) {
		cat("\t", names[j], file=f, append=TRUE, sep="")
	}
	cat("\n", file=f, append=TRUE, sep="")
	#write.table(m, file=f, append=TRUE, quote=FALSE, na="NaN",sep="\t", eol="\n", col.names=FALSE, row.names=FALSE)
write.table(m, file=f, append=TRUE, quote=FALSE, na="NaN",sep="\t", col.names=FALSE, row.names=FALSE)
} 